"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.patch = exports.binder = exports.define = exports.bind = void 0;
/** The original native bind function, before {@link patch} wraps it. */
exports.bind = Function.prototype.bind;
/**
 * Attach the `unbounded` property on the `bounded` function.
 * If `unbounded.unbounded` already exists, then it is used instead, to ensure that the `unbounded` property is always the original function.
 * @param bounded the bounded function
 * @param unbounded the original function, before any binding
 */
function define(bounded, unbounded) {
    if (bounded.unbounded !== unbounded) {
        Object.defineProperty(bounded, 'unbounded', {
            value: unbounded.unbounded || unbounded,
            enumerable: false,
            configurable: false,
            writable: false,
        });
    }
    return bounded;
}
exports.define = define;
/**
 * Alternative to `Function.prototype.bind` that sets the `unbounded` property on the bounded function.
 * Types of this are not intelligent, as using the intelligent bind types causes this/unknown incompatibility. This is not important however, as the masquerading binder of {@link patch} has the intelligent types.
 * @example `a.bind(context, arg1, arg2)` is equivalent to `binder.call(a, context, arg1, arg2)`
 * @param this the function that will be bounded
 * @param newThis the new `this` context to attach to the bounded function
 * @param prefilledArguments the arguments to prefill the bounded function with
 */
function binder(newThis) {
    var prefilledArguments = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        prefilledArguments[_i - 1] = arguments[_i];
    }
    var unbounded = this;
    var bounded = exports.bind.call.apply(exports.bind, __spreadArray([unbounded, newThis], __read(prefilledArguments), false));
    return define(bounded, unbounded);
}
exports.binder = binder;
/** Patch the native `Function.prototype.bind` so that it sets the `unbounded` property on the bounded function. */
function patch() {
    if (Function.prototype.bind !== binder) {
        /* eslint no-extend-native:0 */
        Function.prototype.bind = binder;
    }
}
exports.patch = patch;
